module.exports = require('eden-class').extend(function() {
	/* Require
	-------------------------------*/
	var https 		= require('https');
	var fs 			= require('fs');
	var request		= require('./https/request');
	var response	= require('./https/response');
	
	/* Constants
	-------------------------------*/
	/* Public.Properties
	-------------------------------*/
	/* Protected Properties
	-------------------------------*/
	this._host = '127.0.0.1';
	this._port = 1337;
	this._cert = null;
	this._key  = null;
	this._pfx  = null;
	
	this._resource = null;
	
	/* Private Properties
	-------------------------------*/
	var __noop = function() {};
	
	/* Magic
	-------------------------------*/
	/* Public.Methods
	-------------------------------*/
	/**
	 * Connect to server
	 *
	 * @param function
	 * @return this
	 */
	this.connect = function(callback) {
		this.argument().test(1, 'function', 'undef');
		
		callback = callback || __noop;
		
		if(this._resource !== null) {
			var error = new Error('Server has already started.');
			callback(error);
			return this;
		}

		// ssl options
		var options = {};

		// if both key and cert are set
		if(this._key && this._cert) {
			// set ssl key
			options.key  = this._key;
			// set ssl certificate
			options.cert = this._cert;
		} else {
			// set pfx file
			options.pfx  = this._pfx;
		}
		
		// Server instance
		this._resource = https.createServer(options);
		
		// Server Resource
		this._resource.listen(this._port, this._host);
		
		this._resource.on('request', request(https, this));
		this.on('response', response(https, this));
		
		this._resource.on('close', function() {
			this.trigger('disconnected');	
		}.bind(this));
		
		this.on('connected', callback);
		
		return this.trigger('connected', null);
	};
	
	/**
	 * Disconnects the server
	 *
	 * @param function
	 * @return this
	 */
	this.disconnect = function(callback, attempts) {
		this.argument()
			.test(1, 'function', 'undef')
			.test(2, 'int', 'undef');
		
		attempts = attempts || 0;
		callback = callback || __noop;
		
		if(this._resource === null) {
			callback(new Error('Server is already disconnected.'));
			return this;
		}
		
		try {
			this._resource.close(callback);
			this._resource = null;
		} catch(e) {
			//this will happen because the server has not started yet
			//so we need to wait for it to connect before disconnecting
			if(attempts > 10) {
				callback(new Error('Timed out while waiting for the server to start.'));
				return this;
			}
			
			setTimeout(function() {
				this.disconnect(callback, attempts + 1);
			}.bind(this), 10);
		}
		
		return this;
	};
	
	/**
	 * Returns the host
	 *
	 * @return string
	 */
	this.getHost = function() {
		return this._host;
	};
	
	/**
	 * Returns the port
	 *
	 * @return int
	 */
	this.getPort = function() {
		return this._port;
	};
	
	/**
	 * Returns the connection resource
	 *
	 * @return object
	 */
	this.getResource = function() {
		return this._port;
	};
	
	/**
	 * Sets the identifiable host name
	 *
	 * @param 	string
	 * @return 	this
	 */
	this.setHost = function(host) {
		//Argument Testing
		this.argument().test(1, 'string');
		
		this._host = host;
		return this;
	};
	
	/**
	 * Sets the identifiable port
	 *
	 * @param 	string
	 * @return 	this
	 */
	this.setPort = function(port) {
		//Argument Testing
		this.argument().test(1, 'string', 'int');
		
		this._port = port;
		return this;
	};

	/**
	 * Sets the certificate key file path
	 *
	 * @param 	string
	 * @return 	this
	 */
	this.setKeyFile = function(path) {
		//Argument Testing
		this.argument().test(1, 'string');

		this._key = fs.readFileSync(path);
		return this;
	};

	/**
	 * Sets the certificate file path
	 *
	 * @param 	string
	 * @return 	this
	 */
	this.setCertificateFile = function(path) {
		//Argument Testing
		this.argument().test(1, 'string');

		this._cert = fs.readFileSync(path);
		return this;
	};

	/**
	 * Sets the PFX file (Personal Information Exchange)
	 *
	 * @param 	string
	 * @param 	this
	 */
	this.setPfxFile = function(path) {
		//Argument Testing
		this.argument().test(1, 'string');

		this._pfx = fs.readFileSync(path);
		return this;
	};

	/* Protected Methods
	-------------------------------*/
	/* Private Methods
	-------------------------------*/
}).register('eden/https');